// Copyright (C) 2003 Mooffie <mooffie@typo.co.il>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.

#include <config.h>

#include <unistd.h> // _POSIX_VDISABLE
#include <signal.h>
#include <termios.h>
#include <stdlib.h> // getenv
#include <string.h> // strstr
#ifdef HAVE_LANGINFO_CODESET
# include <langinfo.h>
#endif

#include "terminal.h"
#include "editor.h" // for SIGHUP's emergency_save
#include "dbg.h"

bool terminal::is_utf8;
bool terminal::is_iso88598;
bool terminal::is_fixed;
bool terminal::is_color;
bool terminal::do_arabic_shaping;

static termios oldterm;

RETSIGTYPE terminal::finish(int sig)
{
    endwin();
    tcsetattr(0, TCSANOW, &oldterm);
    DBG(1, ("Bailing out\n"));
    exit(0);
}

RETSIGTYPE terminal::sighup_hndlr(int sig)
{
    Editor::get_global_instance()->emergency_save();
    DBG(1, ("SIGHUP HANDLER\n"));
    exit(0);
}

void terminal::init()
{
    tcgetattr(0, &oldterm);

#ifdef _POSIX_VDISABLE
    termios term;
    term = oldterm;
    term.c_cc[VINTR] = _POSIX_VDISABLE;
    term.c_cc[VQUIT] = _POSIX_VDISABLE;
    term.c_cc[VSTOP] = _POSIX_VDISABLE;
    term.c_cc[VSTART] = _POSIX_VDISABLE;
    tcsetattr(0, TCSANOW, &term);
#else
    termios term;
    term = oldterm;
    term.c_lflag &= ~ISIG;
    term.c_iflag &= ~(IXON | IXOFF);
    tcsetattr(0, TCSANOW, &term);
#endif 
    
    signal(SIGINT, terminal::finish);
    signal(SIGTERM, terminal::finish);
    signal(SIGHUP, terminal::sighup_hndlr);

    // it's important to ignore SIGPIPE because the editor has the ability
    // to write to a pipe.
    signal(SIGPIPE, SIG_IGN);

    initscr();
    keypad(stdscr, TRUE);
    nonl();
    cbreak();
    noecho();

#ifdef HAVE_COLOR
    if (has_colors()) {
	is_color = true;
	start_color();
	
	int bg = COLOR_BLACK;
#ifdef HAVE_USE_DEFAULT_COLORS
	if (use_default_colors() == OK)
	    bg = -1;
#endif

        init_pair(1, COLOR_RED,     bg);
        init_pair(2, COLOR_GREEN,   bg);
        init_pair(3, COLOR_YELLOW,  bg);
        init_pair(4, COLOR_BLUE,    bg);
        init_pair(5, COLOR_CYAN,    bg);
        init_pair(6, COLOR_MAGENTA, bg);
        init_pair(7, COLOR_WHITE,   bg);
    } else {
	is_color = false;
    }
#else
	is_color = false;
#endif

    terminal::do_arabic_shaping = false;
}

// determine_locale() - find out the currently used locale.

void terminal::determine_locale()
{
    const char *locale;
    (void) (((locale = getenv("LC_ALL")) && *locale) ||
    ((locale = getenv("LC_CTYPE")) && *locale) ||
    ((locale = getenv("LANG"))));

    is_utf8 = false;
    is_iso88598 = false;

#ifdef HAVE_LANGINFO_CODESET
# define lcl(subs) strstr(nl_langinfo(CODESET), subs)
#else
# define lcl(subs) (locale && strstr(locale, subs))
#endif

    if (lcl("UTF-8") || lcl("utf-8") || lcl("UTF8") || lcl("utf8"))
        is_utf8 = true;
    if (lcl("8859-8") || lcl("88598"))
	is_iso88598 = true;

    // we assume this is a non-fixed terminal if and only if we are in
    // UTF-8 locale and a DISPLAY environment variable is present (the
    // user can change this with command-line options).
    is_fixed = !(is_utf8 && getenv("DISPLAY"));
}

